/*    (c) 2010 Jan Dlabal <dlabaljan@gmail.com>                               */
/*                                                                            */
/*     This file is part of QuickShare.                                       */
/*                                                                            */
/*     QuickShare is free software: you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by   */
/*     the Free Software Foundation, either version 3 of the License, or      */
/*     any later version.                                                     */
/*                                                                            */
/*     QuickShare is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*     GNU General Public License for more details.                           */
/*                                                                            */
/*     You should have received a copy of the GNU General Public License      */
/*     along with QuickShare. If not, see <http://www.gnu.org/licenses/>.     */


#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <stdio.h>
#endif
#include "qs.h"


GtkWidget *window;
GtkWidget *mbook;
GtkStatusIcon *status_icon;
GtkWidget *lbl_state;
GtkWidget *port_spin;
GtkWidget *btn_launch;
GtkWidget *folder_select;
GtkWidget *iplist_select;
GtkWidget *user_entry;
GtkWidget *pwd_entry;
GtkWidget *upload_spin;
GtkWidget *logtxt;
GtkWidget *icon_menu = NULL;
GtkWidget *icon_menu_onoff;


#ifdef WIN32

char *w32_logf = NULL;
char *w32_logcmd = NULL;

void w32_logv(GtkWidget *widget, gpointer trash)
{
  system(w32_logcmd);
}

void w32_cleanup()
{
  remove(w32_logf);
}

#endif


int ipstate = ALLOW_FILE;
void scrollprint(char *format, ...);


void switch_ipstate(GtkWidget *block, gpointer trash)
{
  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON (block))==TRUE)
  {
    ipstate = BLOCK_FILE;
  } else {
    ipstate = ALLOW_FILE;
  }
}


void switch_window(GtkWidget *widget, gpointer trash)
{
  static char hidden = 0;
  if (!hidden)
    gtk_widget_hide_all(window);
  else
  {
    gtk_widget_show_all(window);
    gtk_notebook_set_current_page(GTK_NOTEBOOK (mbook), 0);
  }
  hidden = hidden == TRUE ? FALSE : TRUE;
}


void icon_popup(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer trash)
{
  gtk_widget_show_all(icon_menu);

  gtk_menu_popup(GTK_MENU (icon_menu),
		 NULL,
		 NULL,
		 gtk_status_icon_position_menu,
		 status_icon,
		 button,
		 activate_time);
}


void upload_changed(GtkWidget *widget, gpointer trash)
{
  if (gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (upload_spin))!=0)
  {
    char *base = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (folder_select));
    char *filename = malloc(strlen(base)+1+strlen("__test__")+1);
    if (!filename)
    {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON (upload_spin), 0);
      notify_error("Unexpected error (not enough memory) occured when testing write permission to upload directory.\nDisabling uploads.");
      return;
    }
    sprintf(filename,"%s%c%s",base,PATH_SEP_CHAR,"__test__");
    g_free(base);
    FILE *fr = g_fopen(filename,"w");
    if (!fr)
    {
      free(filename);
      notify_error("Cannot write to the selected directory. Check if you have write permission or change your selection.\nDisabling uploads.");
      gtk_spin_button_set_value(GTK_SPIN_BUTTON (upload_spin), 0);
      return;
    }
    fclose(fr);
    remove(filename);
    free(filename);
  }
}


int main(int argc, char *argv[])
{
#ifndef WIN32 // Glib threads are not very stable on win32
  g_thread_init(NULL);
  gdk_threads_init();
#endif
  gtk_init(&argc,&argv);
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (window), "QuickShare v1.0");
  gtk_window_set_resizable(GTK_WINDOW (window), FALSE);
  g_signal_connect(G_OBJECT (window), "delete_event",
		   G_CALLBACK (gtk_main_quit), NULL);
  g_signal_connect(G_OBJECT (window), "destroy",
		   G_CALLBACK (gtk_main_quit), NULL);
  mbook = gtk_notebook_new();
  GtkWidget *mbox = gtk_vbox_new(FALSE, 0);
  GtkWidget *basicbox = gtk_vbox_new(FALSE, 0);
  GtkWidget *extbox = gtk_vbox_new(FALSE, 0);
#ifndef WIN32
  GtkWidget *logwin = gtk_scrolled_window_new(FALSE, FALSE);
  logtxt = gtk_text_view_new();
  gtk_text_view_set_editable(GTK_TEXT_VIEW (logtxt), FALSE);
  gtk_container_add(GTK_CONTAINER (logwin), logtxt);
#else
  GtkWidget *logwin = gtk_vbox_new(FALSE, 0);
  GtkWidget *logview = gtk_button_new_with_label("View log file");
  g_signal_connect(G_OBJECT (logview), "clicked", G_CALLBACK (w32_logv), NULL);
  gtk_box_pack_start(GTK_BOX (logwin), logview, FALSE, FALSE, 4);
#endif
  gtk_notebook_append_page(GTK_NOTEBOOK (mbook), basicbox, gtk_label_new("Basic"));
  gtk_notebook_append_page(GTK_NOTEBOOK (mbook), extbox, gtk_label_new("Advanced"));
  gtk_notebook_append_page(GTK_NOTEBOOK (mbook), logwin, gtk_label_new("Log"));
  gtk_notebook_append_page(GTK_NOTEBOOK (mbook), gtk_label_new("QuickShare is (c) 2010, Jan Dlabal.\nThis program is provided under the GNU GPL v3 License.\nHomepage : http://houbysoft.com\nQuickShare : http://houbysoft.com/qs/"), gtk_label_new("About"));
  gtk_container_add(GTK_CONTAINER (window), mbox);
  gtk_box_pack_start_defaults(GTK_BOX (mbox), mbook);
  btn_launch = gtk_button_new_with_label("Start server");
  GtkWidget *statebox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start_defaults(GTK_BOX (statebox), gtk_label_new("Server state : "));
  lbl_state = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL (lbl_state), "<span color=\"red\">Not running</span>");
  gtk_box_pack_start_defaults(GTK_BOX (statebox), lbl_state);
  gtk_box_pack_start_defaults(GTK_BOX (basicbox), btn_launch);
  GtkWidget *lbl_port = gtk_label_new("Run on port : ");
  port_spin = gtk_spin_button_new_with_range(1,65535,1);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON (port_spin), 8080);
  GtkWidget *portbox = gtk_hbox_new(TRUE, 0);
  gtk_box_pack_start_defaults(GTK_BOX (portbox), lbl_port);
  gtk_box_pack_start_defaults(GTK_BOX (portbox), port_spin);
  gtk_box_pack_start_defaults(GTK_BOX (basicbox), portbox);
  GtkWidget *folderbox = gtk_hbox_new(TRUE, 0);
  GtkWidget *lbl_folder = gtk_label_new(" Folder to share : ");
  folder_select = gtk_file_chooser_button_new("Select folder to share", GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
  g_signal_connect(G_OBJECT (folder_select), "selection-changed", G_CALLBACK (upload_changed), NULL);
  gtk_box_pack_start_defaults(GTK_BOX (folderbox), lbl_folder);
  gtk_box_pack_start_defaults(GTK_BOX (folderbox), folder_select);
  gtk_box_pack_start_defaults(GTK_BOX (basicbox), folderbox);
  GtkWidget *userbox = gtk_hbox_new(TRUE, 0);
  GtkWidget *lbl_user = gtk_label_new("Username (leave blank for none) : ");
  user_entry = gtk_entry_new();
  gtk_box_pack_start_defaults(GTK_BOX (userbox), lbl_user);
  gtk_box_pack_start_defaults(GTK_BOX (userbox), user_entry);
  gtk_box_pack_start_defaults(GTK_BOX (basicbox), userbox);
  GtkWidget *pwdbox = gtk_hbox_new(TRUE, 0);
  GtkWidget *lbl_pwd = gtk_label_new("Password : ");
  pwd_entry = gtk_entry_new();
  gtk_entry_set_visibility(GTK_ENTRY (pwd_entry), FALSE);
  gtk_box_pack_start_defaults(GTK_BOX (pwdbox), lbl_pwd);
  gtk_box_pack_start_defaults(GTK_BOX (pwdbox), pwd_entry);
  gtk_box_pack_start_defaults(GTK_BOX (basicbox), pwdbox);
  GtkWidget *upbox = gtk_hbox_new(TRUE, 0);
  GtkWidget *lbl_up = gtk_label_new("Max size (MB) of uploads (0 to disable) : ");
  upload_spin = gtk_spin_button_new_with_range(0,999999,1);
  g_signal_connect(G_OBJECT (upload_spin), "value-changed", G_CALLBACK (upload_changed), NULL);
  GtkWidget *ipbox = gtk_hbox_new(TRUE, 0);
  GtkWidget *lbl_ip = gtk_label_new("IP list file : ");
  iplist_select = gtk_file_chooser_button_new("Select text file with IPs, one per line", GTK_FILE_CHOOSER_ACTION_OPEN);
  GtkWidget *ipbehbox = gtk_hbox_new(TRUE, 0);
  GtkWidget *ipblock = gtk_radio_button_new_with_label(NULL,"Blacklist");
  GtkWidget *ipallow = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (ipblock), "Whitelist");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ipallow), TRUE);
  g_signal_connect(G_OBJECT (ipblock), "toggled", G_CALLBACK (switch_ipstate), NULL);
  gtk_box_pack_start_defaults(GTK_BOX (ipbehbox), ipallow);
  gtk_box_pack_start_defaults(GTK_BOX (ipbehbox), ipblock);
  gtk_box_pack_start_defaults(GTK_BOX (ipbox), lbl_ip);
  gtk_box_pack_start_defaults(GTK_BOX (ipbox), iplist_select);
  gtk_box_pack_start_defaults(GTK_BOX (upbox), lbl_up);
  gtk_box_pack_start_defaults(GTK_BOX (upbox), upload_spin);
  gtk_box_pack_start_defaults(GTK_BOX (extbox), upbox);
  gtk_box_pack_start_defaults(GTK_BOX (extbox), ipbox);
  gtk_box_pack_start_defaults(GTK_BOX (extbox), ipbehbox);
  gtk_box_pack_start_defaults(GTK_BOX (mbox), gtk_hseparator_new());
  gtk_box_pack_start_defaults(GTK_BOX (mbox), statebox);
  g_signal_connect(G_OBJECT (btn_launch), "clicked", G_CALLBACK (switch_daemon), NULL);

  gtk_widget_show_all(window);

  icon_menu = gtk_menu_new();
  icon_menu_onoff = gtk_menu_item_new_with_label("Start server");
  g_signal_connect(G_OBJECT (icon_menu_onoff), "activate", G_CALLBACK (switch_daemon), NULL);
  gtk_menu_append(icon_menu,icon_menu_onoff);
  GtkWidget *quit = gtk_menu_item_new_with_label("Quit");
  g_signal_connect(G_OBJECT (quit), "activate", G_CALLBACK (gtk_main_quit), NULL);
  gtk_menu_append(icon_menu,quit);
  status_icon = gtk_status_icon_new_from_file(STATUS_ICON_OFF);
  g_signal_connect(G_OBJECT (status_icon), "activate", G_CALLBACK (switch_window), NULL);
  g_signal_connect(G_OBJECT (status_icon), "popup-menu", G_CALLBACK (icon_popup), NULL);

#ifdef WIN32
  w32_logf = tmpnam(NULL);
  w32_logcmd = malloc(strlen("start notepad ")+strlen(w32_logf)+1);
  sprintf(w32_logcmd,"start notepad %s",w32_logf);
  scrollprint(""); // create the file
  atexit(w32_cleanup);
#endif

  gtk_main();
  return 0;
}


void scrollprint(char *format, ...)
{
#ifndef WIN32
  gdk_threads_enter();
#endif
  va_list args;

  va_start(args, format);
  gchar *line = g_strdup_vprintf(format,args);
  va_end(args);

#ifndef WIN32
  GtkTextIter iter;
  GtkTextBuffer *scroll_buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW (logtxt));
  gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER (scroll_buf), &iter);
  gtk_text_buffer_insert(GTK_TEXT_BUFFER (scroll_buf), &iter, (gchar *) line, strlen(line));
  gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER (scroll_buf), &iter);
  GtkTextMark *mark = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER (scroll_buf), NULL, &iter, FALSE);
  gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (logtxt), mark, 0.0, FALSE, 0.0, 0.0);
  g_free(line);
  gdk_threads_leave();
#else
  FILE *fw = fopen(w32_logf,"a");
  if (fw)
  {
    fwrite(line, 1, strlen(line), fw);
    fclose(fw);
  }
  g_free(line);
#endif
}
