/*    (c) 2010 Jan Dlabal <dlabaljan@gmail.com>                               */
/*                                                                            */
/*     This file is part of QuickShare.                                       */
/*                                                                            */
/*     QuickShare is free software: you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by   */
/*     the Free Software Foundation, either version 3 of the License, or      */
/*     any later version.                                                     */
/*                                                                            */
/*     QuickShare is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*     GNU General Public License for more details.                           */
/*                                                                            */
/*     You should have received a copy of the GNU General Public License      */
/*     along with QuickShare. If not, see <http://www.gnu.org/licenses/>.     */


#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <string.h>
#include <sys/types.h>
#ifndef WIN32
#include <sys/socket.h>
#include <arpa/inet.h>
#else
#include <Ws2tcpip.h>
#endif
#include <dirent.h>
#include <errno.h>
#include "platform.h"
#include <microhttpd.h>
#include "utils.h"
#include "qs.h"


#define GET 0
#define POST 1


const char *completepage = "<html><head><title>Complete</title></head><body>Upload was succesfully completed.</body></html>";


struct MHD_Daemon *qs_daemon = NULL;
struct connection_info_struct
{
  int connectiontype;
  struct MHD_PostProcessor *postprocessor;
  FILE *fp;
  const char *answerstring;
  char *url;
  int answercode;
  unsigned long long int bytes;
};

long get_file_size (const char *filename)
{
  FILE *fp;
  fp = g_fopen (filename, "rb");
  if (fp)
  {
    long size;
    if ((0 != fseek (fp, 0, SEEK_END)) || (-1 == (size = ftell (fp))))
      size = 0;
    fclose (fp);
    return size;
  }
  else
    return 0;
}

char *strreplace(char *in, char *old, char *new)
{
  char *p = strstr(in,old);
  char *result;
  if (!p)
  {
    result = strdup(in);
    if (!result)
      return NULL;
    return result;
  } else {

    if (TRUE)
    {
      result = malloc(sizeof(char) * (strlen(in)+1-strlen(old)+strlen(new)));
      if (!result)
	return NULL;
      memset(result,0,strlen(in)+1-strlen(old)+strlen(new));
      strncpy(result,in,p-in);
      strcat(result,new);
      p+= strlen(old);
      strcat(result,p);
      char *tmp=malloc(2);
      strcpy(tmp,"#");
      while (strstr(result,old)!=NULL && strcmp(tmp,result)!=0)
      {
        if (tmp)
          free(tmp);
	tmp = result;
	result = strreplace(result,old,new);
      }
      free(tmp);
      return result;
    } else {
      int result_malloc = sizeof(char)*(strlen(in)-strlen(p+1)+1);
      result = malloc(result_malloc);
      if (!result)
	return NULL;
      strncpy(result,in,(strlen(in)-strlen(p+1)));
      result[(strlen(in)-strlen(p+1))]=0;
      char *ptr = in+sizeof(char)*strlen(in)-strlen(p+1);
      while (isalpha((char)(ptr-sizeof(char))[0]) && isalpha(ptr[0]))
      {
	char apme[2];
	apme[0] = ptr[0];
	apme[1] = 0;
	result = realloc(result,++result_malloc);
	strcat(result,apme);
	ptr++;
      }
      char *cat = strreplace(ptr,old,new);
      char *tmp = malloc(sizeof(char) * (strlen(cat)+strlen(result)+1));
      if (!tmp)
	return NULL;
      strcpy(tmp,result);
      strcat(tmp,cat);
      free(result);
      free(cat);
      return tmp;
    }
  }
}

char *iplist = NULL;
int init=0;


int quick_error(struct MHD_Connection *connection, int error)
{
  struct MHD_Response *response;
  // cannot automate since cannot translate automagically between error number and MHD's status codes -- and cannot rely on MHD_HTTP_ values to provide an upper bound for an array
  const char *page_400 = "<html><head><title>Error 400</title></head><body><h1>Error 400 - Bad Request</h1></body></html>";
  const char *page_403 = "<html><head><title>Error 403</title></head><body><h1>Error 403 - Forbidden</h1></body></html>";
  const char *page_404 = "<html><head><title>Error 404</title></head><body><h1>Error 404 - Not Found</h1></body></html>";
  const char *page_500 = "<html><head><title>Error 500</title></head><body><h1>Error 500 - Internal Server Error. Oh no!</body></html>";
  const char *page_501 = "<html><head><title>Error 501</title></head><body><h1>Error 501 - Not Implemented</h1></body></html>";

  int ret;

  switch (error)
  {
  case 400:
    response = MHD_create_response_from_data(strlen(page_400), (char *)page_400, MHD_NO, MHD_NO);
    ret = MHD_queue_response(connection, MHD_HTTP_BAD_REQUEST, response);
    break;

  case 403:
    response = MHD_create_response_from_data(strlen(page_403), (char *)page_403, MHD_NO, MHD_NO);
    ret = MHD_queue_response(connection, MHD_HTTP_FORBIDDEN, response);
    break;

  case 404:
    response = MHD_create_response_from_data(strlen(page_404), (char *)page_404, MHD_NO, MHD_NO);
    ret = MHD_queue_response(connection, MHD_HTTP_NOT_FOUND, response);
    break;

  case 500:
    response = MHD_create_response_from_data(strlen(page_500), (char *)page_500, MHD_NO, MHD_NO);
    ret = MHD_queue_response(connection, MHD_HTTP_INTERNAL_SERVER_ERROR, response);
    break;

  case 501:
    response = MHD_create_response_from_data(strlen(page_501), (char *)page_501, MHD_NO, MHD_NO);
    ret = MHD_queue_response(connection, MHD_HTTP_NOT_IMPLEMENTED, response);
    break;
  }

  MHD_destroy_response(response);
  return ret;
}


int iterate_post (void *coninfo_cls, enum MHD_ValueKind kind, const char *key, const char *filename_relative, const char *content_type, const char *transfer_encoding, const char *data, uint64_t off, size_t size)
{
  FILE *fp;
  struct connection_info_struct *con_info =
    (struct connection_info_struct *) coninfo_cls;

  if (strstr(filename_relative,PATH_SEPARATOR)!=NULL)
  {
    con_info->answerstring = "<html><head><title>Invalid input</title></head><body>Invalid filename. Filenames cannot contain path separators.</body></html>";
    con_info->answercode = MHD_HTTP_FORBIDDEN;
    return MHD_NO;
  }

  con_info->answerstring = "<html><head><title>Error 500</title></head><body><h1>Error 500 - Internal Server Error. Oh no!</h1></body></html>";
  con_info->answercode = MHD_HTTP_INTERNAL_SERVER_ERROR;

  if (strcmp(key, "file") != 0)
    return MHD_NO;

  char *base = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (folder_select));
  char *filename = malloc(strlen(base)+1+strlen(filename_relative)+1);
  if (!filename)
    return MHD_NO;
  sprintf(filename,"%s%c%s",base,PATH_SEP_CHAR,filename_relative);
  g_free(base);

  con_info->bytes += size;
  if (con_info->bytes > (gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (upload_spin))*1024*1024))
  {
    con_info->answerstring = "<html><head><title>File too large</title></head><body><h1>Error : file too large</h1>Please set the maximum filesize to a higher value in QuickShare.</body></html>";
    con_info->answercode = MHD_HTTP_OK;
    remove(filename);
    free(filename);
    return MHD_NO;
  }

  if (!con_info->fp)
  {
    if (fp = g_fopen (filename, "r"))
    {
      fclose (fp);
      con_info->answerstring = "<html><head><title>Error 403</title></head><body><h1>Error 403 - Forbidden (file with same name already exists)</h1></body></html>";
      con_info->answercode = MHD_HTTP_FORBIDDEN;
      free(filename);
      return MHD_NO;
    }

    con_info->fp = g_fopen(filename, "ab");
    if (!con_info->fp)
    {
      free(filename);
      return MHD_NO;
    }
  }

  if (size > 0)
  {
    if (!fwrite (data, size, sizeof (char), con_info->fp))
    {
      free(filename);
      return MHD_NO;
    }
  }

  con_info->answerstring = "<html><head><title>Complete</title></head><body>Upload was succesfully completed.<br><a href='/qsupload'>Upload more files</a> <a href='/'>/</a></body></html>";
  con_info->answercode = MHD_HTTP_OK;
  scrollprint("UPLOAD %s\n",filename);
  free(filename);
  return MHD_YES;
}


int ask_for_authentication (struct MHD_Connection *connection, const char *realm)
{
  int ret;
  struct MHD_Response *response;
  char *headervalue;
  const char *strbase = "Basic realm=";
  
  response = MHD_create_response_from_data (0, NULL, MHD_NO, MHD_NO);
  if (!response) return MHD_NO;
  
  headervalue = malloc (strlen (strbase) + strlen (realm) + 1);
  if (!headervalue) return MHD_NO;  

  strcpy (headervalue, strbase);
  strcat (headervalue, realm);
  
  ret = MHD_add_response_header (response, "WWW-Authenticate", headervalue);
  free (headervalue);  
  if (!ret) {MHD_destroy_response (response); return MHD_NO;}

  ret = MHD_queue_response (connection, MHD_HTTP_UNAUTHORIZED, response);
  
  MHD_destroy_response (response);

  return ret;
}


int is_authed (struct MHD_Connection *connection, const char *username, const char *password)
{
  const char *headervalue;
  char *expected_b64, *expected;
  const char *strbase = "Basic ";
  int authenticated;

  headervalue =
    MHD_lookup_connection_value (connection, MHD_HEADER_KIND,
                                 "Authorization");
  if (NULL == headervalue)
    return 0;
  if (0 != strncmp (headervalue, strbase, strlen (strbase)))
    return 0;

  expected = malloc (strlen (username) + 1 + strlen (password) + 1);
  if (NULL == expected)
    return 0;

  strcpy (expected, username);
  strcat (expected, ":");
  strcat (expected, password);

  expected_b64 = string_to_base64 (expected);
  if (NULL == expected_b64)
    return 0;

  strcpy (expected, strbase);
  authenticated =
    (strcmp (headervalue + strlen (strbase), expected_b64) == 0);

  free (expected_b64);

  return authenticated;
}


void request_completed (void *cls, struct MHD_Connection *connection, void **con_cls, enum MHD_RequestTerminationCode toe)
{
  struct connection_info_struct *con_info = (struct connection_info_struct *) *con_cls;

  if (!con_info)
    return;

  if (con_info->connectiontype == POST)
  {
    if (con_info->postprocessor)
    {
      MHD_destroy_post_processor (con_info->postprocessor);
    }
    
    if (con_info->fp)
      fclose (con_info->fp);
  }
  
  scrollprint("%s %s\n",con_info->connectiontype == POST ? "POST" : "GET",con_info->url);
  free(con_info->url);
  free (con_info);
  *con_cls = NULL;
}


int answer_to_connection(void *cls, struct MHD_Connection *connection, const char *url, const char *method, const char *version, const char *upload_data, size_t *upload_data_size, void **con_cls)
{
  if (*con_cls == NULL)
  {
    struct connection_info_struct *con_info;
    
    con_info = malloc (sizeof (struct connection_info_struct));
    if (con_info == NULL)
      return MHD_NO;

    con_info->fp = NULL;
    con_info->bytes = 0;
    con_info->url = strdup(url);

    if (strcmp (method, "POST") == 0)
    {
      con_info->postprocessor = MHD_create_post_processor (connection, POSTBUFFERSIZE, iterate_post, (void *) con_info);

      if (NULL == con_info->postprocessor)
      {
	free (con_info);
	return MHD_NO;
      }

      con_info->connectiontype = POST;
      con_info->answercode = MHD_HTTP_OK;
      con_info->answerstring = completepage;
    }
    else
      con_info->connectiontype = GET;
    
    *con_cls = (void *) con_info;
    
    return MHD_YES;
  }
  if (strlen(gtk_entry_get_text(GTK_ENTRY (user_entry))))
  {
    if (!is_authed(connection, gtk_entry_get_text(GTK_ENTRY (user_entry)), gtk_entry_get_text(GTK_ENTRY (pwd_entry))))
      return ask_for_authentication(connection, "This folder shared by QuickShare requires authentication");
  }
  struct MHD_Response *response;
  const char *page_upload = "<html><head><title>QuickShare Uploader</title></head><body><h2>Please select file to upload and then click submit.</h2><form action='/qsupload' method='post' enctype='multipart/form-data'><input name='file' type='file'><br><input type='submit'></form></body></html>";
  const char *page_noupload = "<html><head><title>Uploads disabled</title></head><body>Sorry, uploads are disabled. Set the maximum filesize in QuickShare to a non-zero value to enable.</body></html>";
  int ret;

  if (strcmp(method,"GET")!=0 && strcmp(method,"POST")!=0)
    return quick_error(connection, 501);

  if (strcmp(method,"GET")==0)
  {
    if (!test_url_upload(url))
    {      
      char *base = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (folder_select));
      char *fname_tmp = malloc(strlen(base)+1+strlen(url)+1);
      char *fbuf;
      if (!fname_tmp)
	return quick_error(connection, 500);
      strcpy(fname_tmp,base);
      strcat(fname_tmp,PATH_SEPARATOR);
      strcat(fname_tmp,url);
      
#ifndef WIN32
      char *fname = realpath(fname_tmp, NULL);
#else
      char *fname = malloc(_MAX_PATH);
      if (!fname)
      {
	free(fname_tmp);
	return quick_error(connection, 500);
      }
      fname = _fullpath(fname, fname_tmp, _MAX_PATH );
#endif
      free(fname_tmp);
      if (!fname)
      {
	g_free(base);
	return quick_error(connection, 404);
      }
      if (strstr(fname,base)!=fname || (strlen(fname)!=strlen(base) && ((char *)(fname+strlen(base)-(base[strlen(base)-1]==PATH_SEP_CHAR ? 1 : 0)))[0]!=PATH_SEP_CHAR)) // check if file/dir requested is in the allowed range
      {
	g_free(base);
	free(fname);
	return quick_error(connection, 403);
      }
      
      g_free(base);
      
      GDir *directory = g_dir_open(fname,0,NULL);
      long fsize = get_file_size(fname);
      if (fsize && !directory)
      {
	FILE *fr = g_fopen(fname,"rb");
	if (fr)
	{
	  fbuf = malloc(fsize);
	  if (!fbuf)
	  {
	    free(fname);
	    return quick_error(connection, 500);
	  }
	  if (fread(fbuf, 1, fsize, fr)!=fsize)
	  {
	    free(fname);
	    free(fbuf);
	    return quick_error(connection, 500);
	  }
	  fclose(fr);
	}
	
	response = MHD_create_response_from_data(fsize, (void *)fbuf, MHD_YES, MHD_NO);
	MHD_add_response_header(response, "Content-Type", get_mime_type(fname));
	ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	MHD_destroy_response(response);
	
	free(fname);
	
	return ret;
      } else {
	if (directory)
	{
	  //show listing
	  gchar *p_dir_name = NULL;
	  char *page = malloc(strlen("<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" /><title></title></head><body><a href='/qsupload'>[ Upload file to root directory ]</a><br><h1></h1>")+strlen(url)*2+1);
	  if (!page)
	    return quick_error(connection, 500);
	  if (gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (upload_spin))!=0)
	    sprintf(page,"<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" /><title>%s</title></head><body><a href='/qsupload'>[ Upload file to root directory ]</a><br><h1>%s</h1>",url,url);
	  else
	    sprintf(page,"<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" /><title>%s</title></head><body><h1>%s</h1>",url,url);

	  while (p_dir_name = (gchar *)g_dir_read_name(directory))
	  {
	    if (strcmp(p_dir_name,".")==0 || strcmp(p_dir_name,"..")==0)
	      continue;
	    char *url_tmp = g_strdup_printf("%s/%s",url,p_dir_name);
//	    char *url_concat = g_filename_to_uri(url_tmp,NULL,NULL);
//	    g_free(url_tmp);
//	    url_tmp = url_concat;
	    char *url_concat = strreplace(url_tmp,"+","%2B");
	    free(url_tmp);
	    url_tmp = url_concat;
	    url_concat = strreplace(url_concat,"#","%23");
	    free(url_tmp);
	    page = realloc(page,strlen(page)+strlen("<a href=\"\"></a><br>")+strlen(p_dir_name)+1+strlen(url_concat)+1);
	    if (!page)
	      return quick_error(connection, 500);
	    strcat(page,"<a href=\"");
	    char *ptr_tmp = url_concat;
	    while (ptr_tmp[0])
	    {
	      if (ptr_tmp[0]=='/' && ptr_tmp[1]!='/')
		break;
	      ptr_tmp++;
	    }
	    strcat(page,ptr_tmp);
	    free(url_concat);
	    strcat(page,"\">");
	    strcat(page,p_dir_name);
	    strcat(page,"</a><br>");
	  }
	  page = realloc(page,strlen(page)+strlen("</body></html>")+1);
	  if (!page)
	    return quick_error(connection, 500);
	  strcat(page,"</body></html>");
	  g_dir_close(directory);
	  response = MHD_create_response_from_data(strlen(page), (void *)page, MHD_YES, MHD_NO);
	  ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	  MHD_destroy_response(response);
	  
	  free(fname);
	  
	  return ret;
	} else {
	  FILE *fr = g_fopen(fname,"r");
	  if (fr)
	    fclose(fr);
	  if (errno == EACCES)
	  {
	    free(fname);
	    return quick_error(connection, 403);
	  } else {
	    if (errno == EISDIR)
	    {
/*	      directory = opendir(fname);
	      if (errno == EACCES)
	      {
		free(fname);
		return quick_error(connection, 403);
	      } else {
		free(fname);
		return quick_error(connection, 404);
	      }*/
		  return quick_error(connection, 500);
	    } else {
	      free(fname);
	      return quick_error(connection, 404);
	    }
	  }
	}
      }
    } else {
      // Upload page (/qsupload) was requested
      if (gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (upload_spin))!=0)
      {
	response = MHD_create_response_from_data(strlen(page_upload), (void *)page_upload, MHD_NO, MHD_NO);
	ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	MHD_destroy_response(response);
	return ret;
      } else {
	// Uploads were disabled
	response = MHD_create_response_from_data(strlen(page_noupload), (void *)page_noupload, MHD_NO, MHD_NO);
	ret = MHD_queue_response(connection, MHD_HTTP_OK, response);
	MHD_destroy_response(response);
	return ret;
      }
    }
  } else {
    // POST -- upload
    if (!test_url_upload(url))
    {
      return quick_error(connection, 400);
    } else {
      struct connection_info_struct *con_info = *con_cls;
      if (*upload_data_size != 0)
      {
	MHD_post_process (con_info->postprocessor, upload_data,
			  *upload_data_size);
	*upload_data_size = 0;
	return MHD_YES;
      } else {
	response = MHD_create_response_from_data(strlen(con_info->answerstring), (void *)con_info->answerstring, MHD_NO, MHD_NO);
	ret = MHD_queue_response(connection, con_info->answercode, response);
	MHD_destroy_response(response);
	return ret;
      }
    }
  }
}


char *init_ips(int cantalk)
{
  char *fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (iplist_select));
  char *iplist = NULL;
  if (!fname)
  {
    init = ALLOW_ALL;
    iplist = malloc(1);
    if (!iplist)
      return NULL;
    iplist[0] = 0;
    return iplist;
  } else {
    init = USE_FILE;
    FILE *fr = g_fopen(fname,"r");
    if (!fr)
    {
      if (cantalk)      
	notify_error("Cannot open IP file.");
      else
	printf("Error : Cannot open IP file.\n");
      init = 0;
      return NULL;
    }
    char buf[32];
    int alloc = 0;
    while (fgets(buf,32,fr))
    {
      if (buf[strlen(buf)-1]=='\n')
	buf[strlen(buf)-1] = '\0';
      if (strlen(buf))
      {
	if (!check_ip(buf))
	{
	  char *tmp = malloc(strlen("Invalid line in IP file ().")+strlen(buf)+1);
	  if (!tmp)
	    return NULL; // should not happen
	  sprintf(tmp,"Invalid line in IP file (%s).",buf);
	  if (cantalk)
	    notify_error(tmp);
	  else
	    printf(tmp);
	  free(tmp);
	  init = 0;
	  return NULL;
	} else {
	  iplist = realloc(iplist,alloc + strlen(buf) + 1);
	  strcpy((char *)(iplist + alloc), buf);
	  alloc += strlen(buf) + 1;
	}
      }
    }
    iplist = realloc(iplist, alloc + 1);
    iplist[alloc] = '\0';
    fclose(fr);
    g_free(fname);
    return iplist;
  }
}


int client_check(void *cls, const struct sockaddr_in *addr, socklen_t addrlen)
{
  if (!init)
  {
    if (iplist)
      free(iplist);
    iplist = init_ips(FALSE);
  }
#ifndef WIN32
  char buf[INET_ADDRSTRLEN];
  inet_ntop(AF_INET, (const void *)&(addr->sin_addr), (char *)&buf, INET_ADDRSTRLEN);
#else
  char *buf = inet_ntoa(addr->sin_addr);
#endif
  if (init == ALLOW_ALL)
  {
    scrollprint("Allowed access to %s\n",buf);
    return MHD_YES;
  } else {
    char *ptr = iplist;
    while (ptr[0])
    {
      if (strcmp(ptr,buf)==0)
      {
	if (ipstate == ALLOW_FILE)
	{
	  scrollprint("Allowed access to %s\n",buf);
	  return MHD_YES;
	} else {
	  scrollprint("Denied access to %s\n",buf);
	  return MHD_NO;
	}
      }
      ptr += strlen(ptr) + 1;
    }
    if (ipstate == ALLOW_FILE)
    {
      scrollprint("Denied access to %s\n",buf);
      return MHD_NO;
    } else {
      scrollprint("Allowed access to%s\n",buf);
      return MHD_YES;
    }
  }
}


int launch_daemon()
{
  init = 0;
  if (!(iplist = init_ips(TRUE)))
  {
    notify_error("Error : cannot init IP list.\n");
    return 1;
  }
  if (qs_daemon)
  {
    notify_error("Server is already running!");
    return 1;
  }
  int port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON (port_spin));
  qs_daemon = MHD_start_daemon(MHD_USE_THREAD_PER_CONNECTION, port, (MHD_AcceptPolicyCallback) &client_check, NULL,
			       &answer_to_connection, NULL, 
			       MHD_OPTION_NOTIFY_COMPLETED, request_completed, NULL, NULL);
  if (!qs_daemon)
  {
    notify_error("Error when launching server. If you are using a port with a low number (such as 80), make sure you have administrative privileges.");
    return 1;
  } else {
    char *tmp = g_strdup_printf("<span color=\"green\">Running on port %i</span>",port);
    gtk_label_set_markup(GTK_LABEL (lbl_state), tmp);
    g_free(tmp);
    return 0;
  }
}


int stop_daemon()
{
  if (!qs_daemon)
  {
    notify_error("Server is not running!");
    return 1;
  }
  MHD_stop_daemon(qs_daemon);
  qs_daemon = NULL;
  gtk_label_set_markup(GTK_LABEL (lbl_state), "<span color=\"red\">Not running</span>");
  return 0;
}


int switch_daemon()
{
  int r;
  if (!qs_daemon)
  {
    r = launch_daemon();
    if (r)
      return r;
    gtk_button_set_label(GTK_BUTTON (btn_launch), "Stop server");
    gtk_menu_item_set_label(GTK_MENU_ITEM (icon_menu_onoff), "Stop server");
    gtk_status_icon_set_from_file(GTK_STATUS_ICON (status_icon), STATUS_ICON_ON);
  } else {
    r = stop_daemon();
    if (r)
      return r;
    gtk_button_set_label(GTK_BUTTON (btn_launch), "Start server");
    gtk_menu_item_set_label(GTK_MENU_ITEM (icon_menu_onoff), "Start server");
    gtk_status_icon_set_from_file(GTK_STATUS_ICON (status_icon), STATUS_ICON_OFF);
  }
}

