/*    (c) 2010 Jan Dlabal <dlabaljan@gmail.com>                               */
/*                                                                            */
/*     This file is part of QuickShare.                                       */
/*                                                                            */
/*     QuickShare is free software: you can redistribute it and/or modify     */
/*     it under the terms of the GNU General Public License as published by   */
/*     the Free Software Foundation, either version 3 of the License, or      */
/*     any later version.                                                     */
/*                                                                            */
/*     QuickShare is distributed in the hope that it will be useful,          */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*     GNU General Public License for more details.                           */
/*                                                                            */
/*     You should have received a copy of the GNU General Public License      */
/*     along with QuickShare. If not, see <http://www.gnu.org/licenses/>.     */


#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include "mime_ext.h"


void notify(char *string)
{
  GtkWidget *dialog1 = gtk_message_dialog_new(
			NULL,
			GTK_DIALOG_MODAL,
                        GTK_MESSAGE_INFO,
                        GTK_BUTTONS_OK,
                        string);

  gtk_dialog_run (GTK_DIALOG (dialog1));
  gtk_widget_destroy (dialog1);
}

void notify_error(char *string)
{
  GtkWidget *dialog1 = gtk_message_dialog_new(
			NULL,
                        GTK_DIALOG_MODAL,
                        GTK_MESSAGE_ERROR,
                        GTK_BUTTONS_OK,
                        string);

  gtk_dialog_run (GTK_DIALOG (dialog1));
  gtk_widget_destroy (dialog1);
}


char *string_to_base64 (const char *message)
{
  const char *lookup = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  unsigned long l;
  int i;
  char *tmp;
  size_t length = strlen (message);

  tmp = malloc (length * 2);
  if (NULL == tmp)
    return tmp;

  tmp[0] = 0;

  for (i = 0; i < length; i += 3)
  {
    l = (((unsigned long) message[i]) << 16)
      | (((i + 1) < length) ? (((unsigned long) message[i + 1]) << 8) : 0)
      | (((i + 2) < length) ? ((unsigned long) message[i + 2]) : 0);


    strncat (tmp, &lookup[(l >> 18) & 0x3F], 1);
    strncat (tmp, &lookup[(l >> 12) & 0x3F], 1);

    if (i + 1 < length)
      strncat (tmp, &lookup[(l >> 6) & 0x3F], 1);
    if (i + 2 < length)
      strncat (tmp, &lookup[l & 0x3F], 1);
  }

  if (length % 3)
    strncat (tmp, "===", 3 - length % 3);

  return tmp;
}


char check_ip(char *ip)
{
  // IPv6 is not supported right now
  int i=0;
  int total=0;
  while (ip[i] && total < 4)
  {
    int j = i;
    while (isdigit(ip[i]))
      i++;
    if (i - j > 3)
    {
      return 0;
    }
    total++;
    if (ip[i]!='.' && total!=4)
    {
      return 0;
    }
    i++;
  }
  if (total != 4)
    return 0;
  else
    return 1;
}


const char *get_mime_type(char *fname)
{
  char *ext = strrchr(fname,'.');
  if (!ext)
    return "application/binary";
  else
    ext+=1; // skip the dot
  unsigned int i=0;
  for (; i < MIME_EXTS; i++)
  {
    if (strcmp(ext,mime_ext[i][0])==0)
      return mime_ext[i][1];
  }
  return "application/binary";
}
